@@ -19,6 +19,8 @@
 #include <stdio.h>
 #include <math.h>
 #include <string.h>
+#include <time.h>
+#include <sys/time.h>
 #include "screenhack.h"
 #include "xlockmoreI.h"
 
