@@ -77,10 +77,11 @@ ERROR!  Sorry, xlockmore.h requires ANSI C (gcc, for example.)
 #define MI_BATCHCOUNT(MI)	((MI)->batchcount)
 #define MI_SIZE(MI)		((MI)->size)
 #define MI_IS_DRAWN(MI)		((MI)->is_drawn)
-#define MI_IS_DRAWN(MI)		((MI)->is_drawn)
+#define MI_IS_FPS(MI)		((MI)->fps_p)
 #define MI_NCOLORS(MI)		((MI)->npixels)
 #define MI_NAME(MI)		(progname)
 
+#define MI_COLORMAP(MI)	        (MI_WIN_COLORMAP((MI)))
 #define MI_WIDTH(MI)		(MI_WIN_WIDTH((MI)))
 #define MI_HEIGHT(MI)		(MI_WIN_HEIGHT((MI)))
 #define MI_IS_ICONIC(MI)	(MI_WIN_IS_ICONIC((MI)))
